package com.gic.haoban.manage.service.service.content.impl;

import com.alibaba.fastjson.JSON;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.UniqueIdUtils;
import com.gic.haoban.common.utils.StringUtil;
import com.gic.haoban.manage.api.dto.WxEnterpriseQwDTO;
import com.gic.haoban.manage.api.enums.content.ShareBizType;
import com.gic.haoban.manage.service.config.Config;
import com.gic.haoban.manage.service.pojo.bo.content.ClerkShareLogBO;
import com.gic.haoban.manage.service.pojo.bo.content.GroupMessageInfoBo;
import com.gic.haoban.manage.service.pojo.bo.content.TriggerCustomerDetailBO;
import com.gic.haoban.manage.service.pojo.qo.content.ClerkShareLogQO;
import com.gic.haoban.manage.service.pojo.qo.content.TriggerCustomerDetailQO;
import com.gic.haoban.manage.service.service.content.ClerkShareLogService;
import com.gic.haoban.manage.service.service.content.GroupMessageService;
import com.gic.haoban.manage.service.service.content.TriggerCustomerDetailService;
import com.gic.wechat.api.dto.qdto.group.QywxGroupMessageQDTO;
import com.gic.wechat.api.dto.qdto.group.QywxGroupMsgSendResultQDTO;
import com.gic.wechat.api.dto.qywx.group.QywxGroupMessageInfoDTO;
import com.gic.wechat.api.dto.qywx.group.QywxGroupMsgResp;
import com.gic.wechat.api.dto.qywx.group.QywxGroupMsgSendResultDTO;
import com.gic.wechat.api.service.qywx.QywxSuiteApiService;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @Author MUSI
 * @Date 2023/3/19 9:49 PM
 * @Description
 * @Version 企微群发消息处理器
 **/
@Component
public class GroupMessageServiceImpl implements GroupMessageService {

    private static final Logger log = LoggerFactory.getLogger(GroupMessageServiceImpl.class);

    public enum GroupMsgChatType {
        single("single", "发送给客户"),
        group("group", "发送给客户群");

        GroupMsgChatType(String code, String desc) {
            this.code = code;
            this.desc = desc;
        }

        private String code;
        private String desc;

        public String getCode() {
            return code;
        }

        public String getDesc() {
            return desc;
        }
    }

    @Autowired
    private TriggerCustomerDetailService triggerCustomerDetailService;
    @Autowired
    private QywxSuiteApiService qywxSuiteApiService;
    @Autowired
    private ClerkShareLogService clerkShareLogService;

    @Autowired
    private Config config;

    @Override
    public void handlerGroupMessage(GroupMessageInfoBo groupMessageInfoBo) {
        // 获取企业下(当前时间 - 30, 当前时间) 的导购分享日志
        ClerkShareLogQO clerkShareLogQO = new ClerkShareLogQO()
                .setEnterpriseId(groupMessageInfoBo.getEnterpriseId())
                .setStartTime(groupMessageInfoBo.getStartTime())
                .setEndTime(groupMessageInfoBo.getEndTime())
                .setBizType(ShareBizType.MATERIAL.getCode());
        List<ClerkShareLogBO> shareLogBos = clerkShareLogService.queryClerkShareLog(clerkShareLogQO);
        if (CollectionUtils.isEmpty(shareLogBos)) {
            return;
        }
        // 获取企业下的群发记录
        String corpId = "";
        // 发送给客户的消息
        List<QywxGroupMessageInfoDTO> sendWithCustomerMsg = this.queryGroupMsg(corpId, GroupMsgChatType.single);
        // 发送到群
        List<QywxGroupMessageInfoDTO> sendWithGroupMsg = this.queryGroupMsg(corpId, GroupMsgChatType.single);
        // 匹配msgId

        // 获取触发日志详情

        // 保存客户触达日志明细
    }

    private List<QywxGroupMessageInfoDTO> queryGroupMsg(String corpid, GroupMsgChatType groupMsgChatType) {
        List<QywxGroupMessageInfoDTO> groupMessageInfos = new ArrayList<>();
        QywxGroupMessageQDTO groupMessageQDTO = new QywxGroupMessageQDTO();
        groupMessageQDTO.setCorpid(corpid);
        groupMessageQDTO.setSuiteid(config.getWxSuiteid());
        groupMessageQDTO.setChatType(groupMsgChatType.getCode());
        groupMessageQDTO.setPageSize(100);
        ServiceResponse<QywxGroupMsgResp> serviceResponse =
                qywxSuiteApiService.queryQywxGroupMessageList(groupMessageQDTO);
        if (!serviceResponse.isSuccess() || CollectionUtils.isEmpty(serviceResponse.getResult().getGroupMessageInfos())) {
            log.info("获取群发任务记录异常 {}", JSON.toJSONString(serviceResponse));
            return Collections.emptyList();
        }
        QywxGroupMsgResp groupMsgResp = serviceResponse.getResult();
        List<QywxGroupMessageInfoDTO> tempMessageInfos = groupMsgResp.getGroupMessageInfos();
        groupMessageInfos.addAll(tempMessageInfos);
        String cursor = groupMsgResp.getCursor();
        int loopTimes = 0;
        while (StringUtils.isNotBlank(cursor)) {
            loopTimes++;
            log.info("查询群消息 第{}次查询", loopTimes);
            groupMessageQDTO.setCursor(cursor);
            ServiceResponse<QywxGroupMsgResp> tempResponse =
                    qywxSuiteApiService.queryQywxGroupMessageList(groupMessageQDTO);
            if (!tempResponse.isSuccess() || CollectionUtils.isEmpty(tempResponse.getResult().getGroupMessageInfos())) {
                log.info("查询群消息异常 跳出循环 {}", loopTimes);
                break;
            }
            groupMessageInfos.addAll(tempResponse.getResult().getGroupMessageInfos());
            cursor = tempResponse.getResult().getCursor();
        }
        return groupMessageInfos;
    }

    /**
     * 群发给客户
     * @param corpid
     * @param msgId
     * @param userId
     * @param cursor
     * @param clerkShareLogBO
     */
    private void handlerGroupMsgWithCustomer(String corpid, String msgId, String userId, String cursor, ClerkShareLogBO clerkShareLogBO) {

        QywxGroupMsgSendResultQDTO searchQDTO = new QywxGroupMsgSendResultQDTO();
        searchQDTO.setCorpid(corpid);
        searchQDTO.setSuiteid(config.getWxSuiteid());
        searchQDTO.setPageSize(1000);
        searchQDTO.setMsgId(msgId);
        searchQDTO.setUserId(userId);
        searchQDTO.setCursor(cursor);
        ServiceResponse<QywxGroupMsgSendResultDTO> serviceResponse = qywxSuiteApiService.queryQywxGroupMsgSendResult(searchQDTO);
        if (!serviceResponse.isSuccess() || CollectionUtils.isEmpty(serviceResponse.getResult().getSendResultInfos())) {
            return;
        }
        List<TriggerCustomerDetailBO> customerDetailBos = serviceResponse.getResult()
                .getSendResultInfos()
                .stream()
                .map(item -> {
                    TriggerCustomerDetailBO temp = new TriggerCustomerDetailBO();
                    temp.setId(UniqueIdUtils.uniqueLong());
                    temp.setEnterpriseId(clerkShareLogBO.getEnterpriseId());
                    temp.setWxEnterpriseId(clerkShareLogBO.getWxEnterpriseId());
                    temp.setBizId(clerkShareLogBO.getBizId());
                    temp.setBizType(clerkShareLogBO.getBizType());
                    temp.setClerkId(clerkShareLogBO.getClerkId());
                    temp.setStaffId(clerkShareLogBO.getStaffId());
                    temp.setChannelType(clerkShareLogBO.getShareType());
                    temp.setCustomerId(item.getExternalUserId());
                    temp.setCreateTime(new Date());
                    temp.setUpdateTime(new Date());
                    temp.setDeleteFlag(0);
                    return temp;
                })
                .collect(Collectors.toList());
        triggerCustomerDetailService.batchSave(customerDetailBos);

        if (StringUtils.isNotBlank(serviceResponse.getResult().getCursor())) {
            String tempCursor = serviceResponse.getResult().getCursor();
            this.handlerGroupMsgWithCustomer(corpid, msgId, userId, tempCursor, clerkShareLogBO);
        }
        // 标记该条分享日志为已分享
    }
}
